--
-- TriolietFutter
--
-- M@D Author:  Heady
-- M@D date: 24.11.2010
-- Bearbeitet von: pAre
-- nderungsdatum: 24.03.2011
--
-- > Copyright (C) Heady - www.planet-ls.de < --
--

TriolietFutter = {};

function TriolietFutter.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function TriolietFutter:load(xmlFile)

	self.updateSendEvent = SpecializationUtil.callSpecializationsFunction("updateSendEvent");

	local triolietfutter = {};
    triolietfutter.name = getXMLString(xmlFile, "vehicle.triolietfutter#name");
    triolietfutter.openSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.triolietfutter#upSpeedScale"), 1);
    triolietfutter.closeSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.triolietfutter#downSpeedScale"), -triolietfutter.openSpeedScale);
    if triolietfutter.name ~= nil then
        self.triolietfutter = triolietfutter;
    end;
	self.triolietfutter.active = false;
	
	local klappe = {};
    klappe.name = getXMLString(xmlFile, "vehicle.klappe#name");
    klappe.openSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.klappe#upSpeedScale"), 1);
    klappe.closeSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.klappe#downSpeedScale"), -klappe.openSpeedScale);
    if klappe.name ~= nil then
        self.klappe = klappe;
    end;
	self.klappe.active = false;
	
	
	local i=0;
    while true do
        local key = string.format("vehicle.animations.animation(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local name = getXMLString(xmlFile, key.."#name");
        if name ~= nil then		
			if name == "klappe" then			
				local partKey = key..string.format(".part(%d)", 0);
				local loadTime = getXMLFloat(xmlFile, partKey.."#loadTime");
				local endTime = getXMLFloat(xmlFile, partKey.."#endTime");
				
				local currentTime = self:getAnimationTime(self.klappe.name);
				local speed = 1;
				if currentTime > loadTime/endTime then
					speed = -1;
				end;
				self:playAnimation(self.klappe.name, speed, currentTime, true);
				self:setAnimationStopTime(self.klappe.name, loadTime/endTime);
				AnimatedVehicle.updateAnimations(self, 99999999);				
			end;
        end;
        i = i+1;
    end;
	
	self.jointmove = false;

	
end;

function TriolietFutter:delete()
end;

function TriolietFutter:mouseEvent(posX, posY, isDown, isUp, button)
end;

function TriolietFutter:keyEvent(unicode, sym, modifier, isDown)
end;

function TriolietFutter:readStream(streamId, connection)
	-- update animation on synchroninzation --
	local triolietfutterAnimTime = streamReadFloat32(streamId);
	local klappeAnimTime = streamReadFloat32(streamId);
	
	if triolietfutterAnimTime ~= nil then
		local currentTime = self:getAnimationTime(self.triolietfutter.name);
		local speed = 1;
		if currentTime > triolietfutterAnimTime then
			speed = -1;
		end;
		self:playAnimation(self.triolietfutter.name, speed, currentTime, true);
		self:setAnimationStopTime(self.triolietfutter.name, triolietfutterAnimTime);
	end;
	
	if klappeAnimTime ~= nil then
		local currentTime = self:getAnimationTime(self.klappe.name);
		local speed = 1;
		if currentTime > klappeAnimTime then
			speed = -1;
		end;
		self:playAnimation(self.klappe.name, speed, currentTime, true);
		self:setAnimationStopTime(self.klappe.name, klappeAnimTime);	
	end;

	AnimatedVehicle.updateAnimations(self, 50);		

end;

function TriolietFutter:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self:getAnimationTime(self.triolietfutter.name));
	streamWriteFloat32(streamId, self:getAnimationTime(self.klappe.name));
end;

function TriolietFutter:update(dt)
	
	local movejoint = self.jointmove;

	if self:getIsActiveForInput() then	
		
		local isTriolietFutterActive = false;
		if InputBinding.isPressed(InputBinding.TRIOLIET_FUTTERON) then
			self.jointmove = true;		
			isTriolietFutterActive = true;
			if InputBinding.hasEvent(InputBinding.TRIOLIET_FUTTERON) then
				self:playAnimation(self.triolietfutter.name, self.triolietfutter.openSpeedScale, self:getAnimationTime(self.triolietfutter.name)); 
			end;
		elseif InputBinding.isPressed(InputBinding.TRIOLIET_FUTTEROFF) then
			self.jointmove = true;	
			isTriolietFutterActive = true;	
			if InputBinding.hasEvent(InputBinding.TRIOLIET_FUTTEROFF) then
				self:playAnimation(self.triolietfutter.name, self.triolietfutter.closeSpeedScale, self:getAnimationTime(self.triolietfutter.name)); 
			end;			
		end;
			
		if self.triolietfutter.active and not isTriolietFutterActive then
			self:stopAnimation(self.triolietfutter.name);	
			self.jointmove = false;
		end;
		self.triolietfutter.active = isTriolietFutterActive;
		
		
		local isKlappeActive = false;
		if InputBinding.isPressed(InputBinding.TRIOLIET_EXTRADOWN) then
		    self:playAnimation(self.klappe.name, self.klappe.openSpeedScale, self:getAnimationTime(self.klappe.name)); 
			isKlappeActive = true;	
			if InputBinding.hasEvent(InputBinding.TRIOLIET_EXTRADOWN) then
				self:playAnimation(self.klappe.name, self.klappe.openSpeedScale, self:getAnimationTime(self.klappe.name)); 
			end;			
		elseif InputBinding.isPressed(InputBinding.TRIOLIET_EXTRAUP) then
			self:playAnimation(self.klappe.name, self.klappe.closeSpeedScale, self:getAnimationTime(self.klappe.name));
			isKlappeActive = true;
			if InputBinding.hasEvent(InputBinding.TRIOLIET_EXTRAUP) then
				self:playAnimation(self.klappe.name, self.klappe.closeSpeedScale, self:getAnimationTime(self.klappe.name));
			end;
		end
		if self.klappe.active and not isKlappeActive then
      		self:stopAnimation(self.klappe.name);	
		end;
		self.klappe.active = isKlappeActive;
		
	end;
	
	if movejoint ~= self.jointmove then -- wenn gendert dann sende an server/client (minimierung der bertragungsrate) --
		self:updateSendEvent();
	end;
	
	if self:getIsActive() then
	
		if self.isServer then
			if self.attacherVehicle ~= nil and self.jointmove then
				for k, implement in pairs(self.attacherVehicle.attachedImplements) do
					local jointDesc = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
					if k == self.attacherVehicle.selectedImplement  then
						setJointFrame(jointDesc.jointIndex, 1, self.attacherJoint.node);
					end;
				end;
			end;
		end;
		
	end;

end;

function TriolietFutter:updateTick(dt)
	if self:getIsActive() then
		for _, part in pairs(self.movingParts) do
			part.isDirty = true;
		end;	
	end;
end;

function TriolietFutter:draw()	

	g_currentMission:addExtraPrintText(InputBinding.getKeyNamesOfDigitalAction(InputBinding.TRIOLIET_FUTTERON).." / "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.TRIOLIET_FUTTEROFF)..": Trioliet Futter an / aus");

end;

function TriolietFutter:updateSendEvent()	

	if g_server ~= nil then
		g_server:broadcastEvent(MPEvent:new(self));
	else
		g_client:getServerConnection():sendEvent(MPEvent:new(self));
	end;

end;


